function [bus, gen, branch] = opfsoln(baseMVA, bus0, gen0, branch0, ...
								Ybus, Yf, Yt, V, Pg, lamda, ref, pv, pq, alg)
%OTOPFSOL   Updates bus, gen, branch data structures to match opf soln.
%   [bus, gen, branch] = opfsoln(baseMVA, bus0, gen0, branch0, ...
%                              Ybus, Yf, Yt, V, Pg, lamda, ref, pv, pq, alg)

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% constants
j = sqrt(-1);
nl = size(branch0, 1);		%% number of lines

%% define named indices into bus, gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% initialize return values
bus		= bus0;
gen		= gen0;
branch	= branch0;

%%----- update bus voltages -----
bus(:, VM) = abs(V);
bus(:, VA) = angle(V) * 180 / pi;

%%----- update Pg and Qg for all gens -----
%% generator info
on = find(gen(:, GEN_STATUS));				%% which generators are on?
gbus = gen(on, GEN_BUS);					%% what buses are they at?
refgen = find(gen(:, GEN_BUS) == ref);		%% which is the reference gen?

%% copy back Pg & Vg
gen(:, PG) = zeros(size(gen, 1), 1);
gen(on, PG) = Pg * baseMVA;
gen(on, VG) = bus(gbus, VM);

%% compute total injected bus powers
Sg = V(gbus) .* conj(Ybus(gbus, :) * V);

%% update Qg for all generators
gen(:, QG) = zeros(size(gen, 1), 1);				%% zero out all Qg
gen(on, QG) = imag(Sg) * baseMVA + bus(gbus, QD);	%% inj Q + local Qd

%%----- update/compute branch power flows -----
Sf = V(branch(:, F_BUS)) .* conj(Yf * V) * baseMVA;	%% complex power at "from" bus
St = V(branch(:, T_BUS)) .* conj(Yt * V) * baseMVA;	%% complex power injected at "to" bus
branch(:, [PF, QF, PT, QT]) = [real(Sf) imag(Sf) real(St) imag(St)];

%%----- update lamda's and mu's -----
%% compute indexing
nb = size(bus, 1);
nl = size(branch, 1);
npv	= length(pv);
npq	= length(pq);
ng = npv + 1;
i1 = 1;			i2 = ng;			%% i1:i2 - P mismatch gen buses
i3 = i2 + 1;	i4 = i2 + npq;		%% i3:i4 - P mismatch PQ buses
i5 = i4 + 1;	i6 = i4 + npq;		%% i5:i6 - Q mismatch PQ buses
if varVg(alg)
	i7 = i6 + 1;	i8 = i6 + nb;	%% i7:i8 - Vmin all buses
	i9 = i8 + 1;	i10 = i8 + nb;	%% i9:i10 - Vmax all buses
else
	i7 = i6 + 1;	i8 = i6 + npq;	%% i7:i8 - Vmin PQ buses
	i9 = i8 + 1;	i10 = i8 + npq;	%% i9:i10 - Vmax PQ buses
end
i11 = i10 + 1;	i12 = i10 + ng;		%% i11:i12 - Pmin gen buses
i13 = i12 + 1;	i14 = i12 + ng;		%% i13:i14 - Pmax gen buses
i15 = i14 + 1;	i16 = i14 + ng;		%% i15:i16 - Qmin gen buses
i17 = i16 + 1;	i18 = i16 + ng;		%% i17:i18 - Qmax gen buses
i19 = i18 + 1;	i20 = i18 + nl;		%% i19:i20 - |Sf| line limit
i21 = i20 + 1;	i22 = i20 + nl;		%% i21:i22 - |St| line limit

%% initialize all with zeros
bus(:, [LAM_P, LAM_Q, MU_VMIN, MU_VMAX]) = zeros(nb, 4);
gen(:, [MU_PMIN, MU_PMAX, MU_QMIN, MU_QMAX]) = zeros(size(gen, 1), 4);
branch(:, [MU_SF, MU_ST]) = zeros(nl, 2);

%% copy multipiers to bus, gen, branch
bus(gbus, LAM_P)	= lamda(i1:i2) / baseMVA;
bus(pq, LAM_P)		= lamda(i3:i4) / baseMVA;
bus(pq, LAM_Q)		= lamda(i5:i6) / baseMVA;
if varVg(alg)
	bus(:, MU_VMIN)		= lamda(i7:i8);
	bus(:, MU_VMAX)		= lamda(i9:i10);
else
	bus(pq, MU_VMIN)	= lamda(i7:i8);
	bus(pq, MU_VMAX)	= lamda(i9:i10);
end

gen(on, MU_PMIN)	= lamda(i11:i12) / baseMVA;
gen(on, MU_PMAX)	= lamda(i13:i14) / baseMVA;
gen(on, MU_QMIN)	= lamda(i15:i16) / baseMVA;
gen(on, MU_QMAX)	= lamda(i17:i18) / baseMVA;

branch(:, MU_SF)	= lamda(i19:i20) / baseMVA;
branch(:, MU_ST)	= lamda(i21:i22) / baseMVA;

%% compute LAM_Q for generator buses
i = find(gen(on, MU_QMIN));
bus(gbus(i), LAM_Q)	= -gen(on(i), MU_QMIN);
i = find(gen(on, MU_QMAX));
bus(gbus(i), LAM_Q)	= gen(on(i), MU_QMAX);

return;
